/*
Itselfcat - An iterated spatial self-categorization simulation program
Copyright (C) 2004 by Laurent Salzarulo
E-mail: Laurent.Salzarulo@unil.ch

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "itselfcat.h"

/* Using an analytic form of the gradient of  */
/* the function to minimize is faster in      */
/* multi-dimensional spaces.                  */
#if (OPINION_DIMENSIONS > 1)
#define USE_GRADIENT_ALGORITHM
#endif

#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <algorithm>
#include <string>
#include <locale>

#ifdef ENABLE_GRAPHICAL_MODE
	#include <gtk/gtk.h>
	#include <gdk/gdkkeysyms.h>
	#include <gdk/gdk.h>
	#include <gdk-pixbuf/gdk-pixbuf.h>
#endif

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_rng.h>

using namespace std;

static double a;
static double w;
/* width of the cells (in pixels) */
static const int CELLSIZE=5;
/* number of actual iteration */
static int iteration;
/* number of actual experiment */
static int experiment;
/* number of performed experiments */
static int computed_experiments;
static double percent_computed_experiments;
static int total_experiments_number;//
/* used to test wether the system reached a stationary state */
bool nothing_happened;

/* Initialization of the GSL minimizer */
#ifdef USE_GRADIENT_ALGORITHM
static const gsl_multimin_fdfminimizer_type *T = gsl_multimin_fdfminimizer_steepest_descent;
static gsl_multimin_fdfminimizer *s = NULL;
static gsl_multimin_function_fdf min_func;
#else
static const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;
static gsl_multimin_fminimizer *s = NULL;
static gsl_multimin_function min_func;
#endif
static gsl_vector *ss, *x;

/* Initialization of the GSL random number generator  */
static const gsl_rng_type * TRand;
static gsl_rng *r;

/* U is the opposite of the prototypicality  */
/* function. Its minima (for *v) are the     */
/* prototypical positions in the context     */
/* *params (which is a list of the opinions  */
/* of the neighbours.                        */
double U(const gsl_vector *v, void *params) {
	double d_intra=0.0;
	double d_inter=0.0;
	double mu=0.0;
	double norm_intra=0.0;
	double dist_mult=0.0;
	/* loop on the neighbours */
	for(int i=0; i< ((gsl_matrix*)params)->size1; i++) {
		dist_mult=0.0;
		/* loop on the opinion dimensions */
		for (int j=0; j< ((gsl_matrix*)params)->size2; j++) {
			dist_mult += pow(gsl_vector_get(v,j) - gsl_matrix_get((gsl_matrix*)params,i,j), 2);
		}
		mu=exp(-dist_mult/pow(w,2));
		d_intra+=mu*dist_mult;
		d_inter+=(1-mu)*dist_mult;
		norm_intra+=mu;
	}
	d_intra/=norm_intra;
	if(((gsl_matrix*)params)->size1 != norm_intra) {d_inter/= ((gsl_matrix*)params)->size1 - norm_intra;}
	else d_inter=0.0; /* in case there is no outgroup */
	return (1-a)*d_intra-a*d_inter;
}

/* derivate of U */
void dU(const gsl_vector *v, void *params, gsl_vector *df) {
	double mu[(int)((gsl_matrix*)params)->size1];
	double d2[(int)((gsl_matrix*)params)->size1];
	double norm_intra=0.0;
	double diff=0.0;
	double di1[(int)((gsl_matrix*)params)->size2];
	double di2[(int)((gsl_matrix*)params)->size2];
	double di3[(int)((gsl_matrix*)params)->size2];
	double dt1[(int)((gsl_matrix*)params)->size2];
	double dt2[(int)((gsl_matrix*)params)->size2];
	double dt3[(int)((gsl_matrix*)params)->size2];
	double dt4[(int)((gsl_matrix*)params)->size2];
	double dd_intra[(int)((gsl_matrix*)params)->size2];
	double dd_inter[(int)((gsl_matrix*)params)->size2];

	/* compute mu(xi) */
	for(int i=0; i< ((gsl_matrix*)params)->size1; i++) {

		d2[i]=0.0;
		/* loop on the opinion dimensions */
		for (int j=0; j< ((gsl_matrix*)params)->size2; j++) {
			d2[i] += pow(gsl_vector_get(v,j) - gsl_matrix_get(((gsl_matrix*)params),i,j), 2);
		}
		mu[i]=exp(-d2[i]/pow(w,2));
		norm_intra+=mu[i];
	}

	/* loop on the components of grad(U) */
	for (int alpha=0; alpha<((gsl_matrix*)params)->size2; alpha++) {
		di1[alpha]=0.0;
		di2[alpha]=0.0;
		di3[alpha]=0.0;
		dt1[alpha]=0.0;
		dt2[alpha]=0.0;
		dt3[alpha]=0.0;
		dt4[alpha]=0.0;
		/* loop on the neighbours */
		for(int i=0; i< ((gsl_matrix*)params)->size1; i++) {
			diff = gsl_vector_get(v,alpha) - gsl_matrix_get(((gsl_matrix*)params),i,alpha);
			di1[alpha]+=diff*d2[i]*mu[i];
			di2[alpha]+=diff*mu[i];
			di3[alpha]+=d2[i]*mu[i];
			dt1[alpha]+=diff*(1.0-mu[i])*d2[i];
			dt2[alpha]+=diff*d2[i];
			dt3[alpha]+=diff*(1.0-mu[i]);
			dt4[alpha]+=(1.0-mu[i])*d2[i];
		}
		di3[alpha]/=norm_intra;
		dd_intra[alpha] = -2*di1[alpha]/pow(w,2)+2*di2[alpha]*(1+di3[alpha]/pow(w,2));
		dd_intra[alpha]/=norm_intra;
		if(((gsl_matrix*)params)->size1 != norm_intra) {
			dt4[alpha]/=((gsl_matrix*)params)->size1 - norm_intra;
			dd_inter[alpha] = -2*dt1[alpha]/pow(w,2)+2*dt2[alpha]/pow(w,2)+2*dt3[alpha]-2*dt4[alpha]*di2[alpha]/pow(w,2);
			dd_inter[alpha]/=((gsl_matrix*)params)->size1 - norm_intra;
		} else {
			dd_inter[alpha]=0.0;
		}
		gsl_vector_set(df,alpha,(1-a)*dd_intra[alpha]-a*dd_inter[alpha]);
	}
}

void UdU (const gsl_vector *x, void *params, double *f, gsl_vector *df) {
	*f=U(x,params);
	dU(x,params,df);
}

class Agent {
	public:
		/* relation contains pointers to the neighbours */
		vector<Agent*> relation;
		/* nb_swn_added_links: number of links created by rewiring process during SWN construction */
		int nb_swn_added_links;
		/* agent's opinion */
		vector<double> opinion;
		/* agent's new opinion (for synchronized update) */
		vector<double> new_opinion;
		/* agent's last opinion (for calculation of the
		 mean laste change of opinion) */
		vector<double> last_opinion;
		/* constructor */
		Agent() {nb_swn_added_links=0;};

		/* computes agent's mean opinion */
		double mean_op();
		/* computes agent's last change of opinion */
		double change_op();
		/* performs self-categorization */
		void self_categorize(gsl_matrix*);
		/* updates opinion */
		void actualize();
};

double Agent::mean_op() {
	double result=0.0;
	for (int c=0; c<OPINION_DIMENSIONS; c++) {
		result+=opinion.at(c);
	}
	result/=OPINION_DIMENSIONS;
	return result;
}

double Agent::change_op() {
	double result=0.0;
	for (int c=0; c<OPINION_DIMENSIONS; c++) {
		result+=fabs(opinion.at(c)-last_opinion.at(c));
	}
	result/=OPINION_DIMENSIONS;
	return result;
}

void Agent::self_categorize(gsl_matrix* op) {
	/* number of variables of function U */
	size_t np= op->size2;
	size_t iter=0;
	int status;
	double size;
	#ifdef USE_GRADIENT_ALGORITHM
	/* set first step size to 0.05 */
	double stepsize=0.05;
	#else
	/* initial vertex size vector */
	ss=gsl_vector_alloc(np);
	/* set all step sizes to 0.05 */
	gsl_vector_set_all(ss,0.05);
	#endif
	/* starting point = opinion of the agent */
	x=gsl_vector_alloc(np);
	for (int j=0; j<op->size2; j++) {
		gsl_vector_set(x,j,opinion.at(j));
	}

	/* initialize method and iterate */
	min_func.f=&U;
	min_func.n=np;
	min_func.params=op;
	#ifdef USE_GRADIENT_ALGORITHM
	min_func.df=&dU;
	min_func.fdf=&UdU;
	s=gsl_multimin_fdfminimizer_alloc(T,np);
	gsl_multimin_fdfminimizer_set(s,&min_func,x,stepsize,0.05);
	#else
	s=gsl_multimin_fminimizer_alloc(T,np);
	gsl_multimin_fminimizer_set(s,&min_func,x,ss);
	#endif

	do {
		iter++;
		#ifdef USE_GRADIENT_ALGORITHM
		status=gsl_multimin_fdfminimizer_iterate(s);
		if (status) break;
		status=gsl_multimin_test_gradient(s->gradient,1e-4);
		#else
		status=gsl_multimin_fminimizer_iterate(s);
		if (status) break;
		size=gsl_multimin_fminimizer_size(s);
		status=gsl_multimin_test_size(size,1e-4);
		#endif
	}
	while (status==GSL_CONTINUE && iter<200);
	/* if the maximum of iteration is reached, the minimum of U */
	/* is outside of [0,1] */
	if(status!=GSL_SUCCESS) {
		/* replace extreme values of minimum of U */
		for (int j=0; j< op->size2; j++) {
			#ifdef USE_GRADIENT_ALGORITHM
			if (gsl_vector_get(gsl_multimin_fdfminimizer_x(s),j)<0) {
				gsl_vector_set(gsl_multimin_fdfminimizer_x(s),j,0);
			}
			if (gsl_vector_get(gsl_multimin_fdfminimizer_x(s),j)>1) {
				gsl_vector_set(gsl_multimin_fdfminimizer_x(s),j,1);
			}
			#else
			cout<<"!"<<endl;
			if (gsl_vector_get(gsl_multimin_fminimizer_x(s),j)<0) {
				gsl_vector_set(gsl_multimin_fminimizer_x(s),j,0);
			}
			if (gsl_vector_get(gsl_multimin_fminimizer_x(s),j)>1) {
				gsl_vector_set(gsl_multimin_fminimizer_x(s),j,1);
			}
			#endif
		}
	}
	new_opinion.clear();
	
	#if (EXISTING_OPINION==0)
	// choose the most prototypical opinion (even if no neighbour has it)
	double newOp;
	for (int j=0; j<op->size2; j++) {
		newOp=gsl_vector_get(s->x,j);
		new_opinion.push_back(newOp);
	}
	#else
	// choose the most prototypical opinion among those of the neighbours
	double dist_neighbour_prototype;
	double dist_the_one_prototype=100.0;
	int the_one=0;
	double op_the_one=100.0;
	// loop on the agents 
	for(int i=0; i< op->size1; i++) {
		dist_neighbour_prototype=0.0;
		// loop on the opinion dimensions 
		for (int j=0; j< op->size2; j++) {
			dist_neighbour_prototype += pow(gsl_vector_get(s->x,j) - gsl_matrix_get(op,i,j), 2);
		}
		if (dist_neighbour_prototype<dist_the_one_prototype) {
			the_one=i;
			dist_the_one_prototype=dist_neighbour_prototype;
			op_the_one=gsl_matrix_get(op,i,0);
		}
	}
	for (int j=0; j<op->size2; j++) {
		new_opinion.push_back(gsl_matrix_get(op,the_one,j));
	}
	#endif

	gsl_vector_free(x);
	#ifdef USE_GRADIENT_ALGORITHM
	gsl_multimin_fdfminimizer_free(s);
	#else
	gsl_vector_free(ss);
	gsl_multimin_fminimizer_free(s);
	#endif
}

void Agent::actualize() { 
	for (int c=0; c<OPINION_DIMENSIONS; c++) {
		last_opinion[c]=opinion[c];
		if(opinion[c]!=new_opinion[c]) {
			opinion[c]=new_opinion[c];
			nothing_happened=false;
		}
	}
}

vector<Agent*> agent;
Agent* cell[GRID_SIZE][GRID_SIZE];

inline int periodicBC(int x) {
	int x_new= x%GRID_SIZE;
	if (x_new<0) {x_new+=GRID_SIZE;}
	return x_new;
}

#ifdef ENABLE_GRAPHICAL_MODE
static GdkPixmap *pixmap = NULL;
static GdkPixmap *pixmap_grad=NULL;
static GdkPixbuf *pixbuf=NULL;
/* info popup */
static GtkWidget *info_popup;
/* opinions' representation window */
static GtkWidget *window=NULL, *drawing_area=NULL, *vbox=NULL;
/* opinion gradient's representation window */
static GtkWidget *window_grad=NULL, *drawing_area_grad=NULL, *vbox_grad=NULL;
void key_event (GtkWidget *widget, GdkEventKey *event, gpointer data);

/* draw all opinions */
static void draw_all( GtkWidget *widget){
	if (pixmap) gdk_pixmap_unref(pixmap);
	pixmap = gdk_pixmap_new(widget->window, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE, -1);
	gdk_draw_rectangle (pixmap, widget->style->white_gc,TRUE,0, 0,GRID_SIZE*CELLSIZE,GRID_SIZE*CELLSIZE);
	GdkRectangle update_rect;
	GdkColor *color;
	GdkGC *gc;
	gc = gdk_gc_new(widget->window);
	color = (GdkColor *)malloc(sizeof(GdkColor));

	for (int i=0; i<GRID_SIZE; i++) {
		for (int j=0; j<GRID_SIZE; j++) {
			/*if there is an agent on (i,j)*/
			if (cell[i][j]!=0) {
				color->red=int(cell[i][j]->mean_op()*65535);
				color->blue=int((1.0-cell[i][j]->mean_op())*65535);
				color->green=0;
			} else { /*no agent on (i,j)*/
				color->red=0;
				color->blue=0;
				color->green=0;
			}
			color->pixel=(gulong)(color->red*65536 + color->green*256 + color->blue);
			gdk_color_alloc(gtk_widget_get_colormap(widget), color);
			gdk_gc_set_foreground(gc, color);
			gdk_draw_rectangle (pixmap, gc, TRUE, i *CELLSIZE, j *CELLSIZE, CELLSIZE, CELLSIZE);
		}
	}
	update_rect.y = 0;
	update_rect.x = 0;
	update_rect.width = GRID_SIZE*CELLSIZE;
	update_rect.height = GRID_SIZE*CELLSIZE;
	gtk_widget_draw (widget, &update_rect);
}

/* draw opinion gradient */
static void draw_all_grad( GtkWidget *widget){
	if (pixmap_grad) gdk_pixmap_unref(pixmap_grad);
	pixmap_grad = gdk_pixmap_new(widget->window, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE, -1);
	gdk_draw_rectangle (pixmap_grad, widget->style->white_gc,TRUE,0, 0,GRID_SIZE*CELLSIZE,GRID_SIZE*CELLSIZE);
	GdkRectangle update_rect;
	GdkColor *color;
	GdkGC *gc;
	gc = gdk_gc_new(widget->window);
	color = (GdkColor *)malloc(sizeof(GdkColor));

	double gradient=0.0;
	Agent *neighbour1, *neighbour2;

	for (int i=0; i<GRID_SIZE; i++) {
		for (int j=0; j<GRID_SIZE; j++) {
			gradient=0.0;
			neighbour1=cell[periodicBC(i-1)][periodicBC(j-1)];
			neighbour2=cell[periodicBC(i)][periodicBC(j)];
			if(neighbour1 != 0 && neighbour2 != 0) {
				gradient += fabs(neighbour1->mean_op() - neighbour2->mean_op());
			}
			neighbour1=cell[periodicBC(i)][periodicBC(j-1)];
			neighbour2=cell[periodicBC(i-1)][periodicBC(j)];
			if(neighbour1 != 0 && neighbour2 != 0) {
				gradient += fabs(neighbour1->mean_op() - neighbour2->mean_op());
			}
			gradient/=2.0;

			color->red=int(gradient*65535);
			color->blue=1;
			color->green=1;
			color->pixel=(gulong)(color->red*65536 + color->green*256 + color->blue);
			gdk_color_alloc(gtk_widget_get_colormap(widget), color);
			gdk_gc_set_foreground(gc, color);
			gdk_draw_rectangle (pixmap_grad, gc, TRUE, i *CELLSIZE, j *CELLSIZE, CELLSIZE, CELLSIZE);
			}
	}
	update_rect.y = 0;
	update_rect.x = 0;
	update_rect.width = GRID_SIZE*CELLSIZE;
	update_rect.height = GRID_SIZE*CELLSIZE;
	gtk_widget_draw (widget, &update_rect);
}

/* redraw the opinion window on creation and resizing */
static gint configure_event( GtkWidget *widget, GdkEventConfigure *event ) {
	if (pixmap) gdk_pixmap_unref(pixmap);
	pixmap = gdk_pixmap_new(widget->window, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE, -1);
	draw_all(drawing_area);
	return TRUE;
}

/* redraw the opinion window on expose event */
static gint expose_event( GtkWidget *widget, GdkEventExpose *event ) {
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);
	return FALSE;
}
/* redraw the gradient window on creation and resizing */
static gint configure_event_grad( GtkWidget *widget, GdkEventConfigure *event ) {
	if (pixmap_grad) gdk_pixmap_unref(pixmap_grad);
	pixmap_grad = gdk_pixmap_new(widget->window, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE, -1);
	draw_all(drawing_area_grad);
	return TRUE;
}

/* redraw the gradient window on expose event */
static gint expose_event_grad( GtkWidget *widget, GdkEventExpose *event ) {
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap_grad, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);
	return FALSE;
}

/* quit if a window is closed */
gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer data ) {
	return(FALSE);
}

/* quit if a window is destroyed */
void destroy( GtkWidget *widget, gpointer data ) {
	gsl_rng_free(r);
	gtk_main_quit();
}
#endif

/* The following procedures compute the different statistics to output */

/* total number of different opinions in the population */
double number_of_different_opinions() {
	int result=1;
	bool different_opinion;
	vector<double> list_of_opinion;
	list_of_opinion.clear();
	list_of_opinion.push_back(agent[0]->mean_op());
	for (int i=1; i<agent.size(); i++) {
		different_opinion=true;
		for(int index=0; index<list_of_opinion.size(); index++) {
			/* opinions are considered as equal within a difference of +/- 0.0001 */
			if (agent[i]->mean_op()>(list_of_opinion[index]-0.001) && agent[i]->mean_op()<(list_of_opinion[index]+0.001)) {
				different_opinion=false;
			}
		}
		if(different_opinion) {
			list_of_opinion.push_back(agent[i]->mean_op());
			result++;
		}
	}
	return result;
}

/* mean opinion in the population */
double opinion_mean() {
	double opinion_sum=0.0;
	for (int i=0; i<agent.size(); i++) {
		opinion_sum+=agent[i]->mean_op();
	}
	return opinion_sum/agent.size();
}

/* variance of opinions in the population */
double opinion_var() {
	double opinion_sum=0.0;
	double square_opinion_sum=0.0;
	for (int i=0; i<agent.size(); i++) {
		opinion_sum+=agent[i]->mean_op();
		square_opinion_sum+=pow(agent[i]->mean_op(),2);
	}
	return square_opinion_sum/agent.size()-pow(opinion_sum/agent.size(),2);
}

/* histogram of opinions in the population */
string opinion_histogram(int nb_class) {
	char result[nb_class*20];
	char buffer[20];
	vector<double> opinion_distribution(nb_class);
	opinion_distribution.clear();
	/* computes the distribution of the opinions within nb_class classes */
	for (int i=0; i< agent.size(); i++) {
		if(agent[i]->mean_op()==1.0) opinion_distribution[nb_class-1]++;
		else opinion_distribution[int(nb_class*(agent[i]->mean_op()))]++;
	}
	opinion_distribution[0]/=agent.size();
	sprintf(result, "%.5f", opinion_distribution[0]);
	for (int index=1; index<nb_class; index++) {
		opinion_distribution[index] /= agent.size();
		sprintf(buffer, "\t%.5f", opinion_distribution[index]);
		strcat(result, buffer);
	}
	return result;
}

/* complete list of opinions in the population */
string opinion_list() {
	char result[GRID_SIZE*GRID_SIZE*8];
	char buffer[8];
	sprintf(result, "%f", agent[0]->mean_op());
	for (int i=1; i<agent.size(); i++) {
		sprintf(buffer, "\t%.4f", agent[i]->mean_op());
		strcat(result, buffer);
	}
	return result;
}

/* mean last change of opinion in the population */
double mean_last_change_of_opinion() {
	double change_sum=0.0;
	for (int i=0; i<agent.size(); i++) {
		change_sum+=agent[i]->change_op();
	}
	return change_sum/agent.size();
}

#ifdef ENABLE_GRAPHICAL_MODE
/* output of a snapshot */
void save_image() {
	setlocale(LC_ALL, "C");
	GError *error=NULL;
	char filename[200];
	#if (MOORE_RADIUS==INF)
	sprintf(filename, "snapshot_a%4.3f_w%4.3f_rinf_p%4.3f_i%d.png", a,w,SMALLWORLD_P,iteration);
	#else
	sprintf(filename, "snapshot_a%4.3f_w%4.3f_r%d_p%4.3f_i%d.png", a, w, int(MOORE_RADIUS), double(SMALLWORLD_P), int(iteration));
	#endif
	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 0, 8, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE);
	gdk_pixbuf_get_from_drawable (pixbuf,pixmap, gtk_widget_get_colormap(drawing_area),0, 0, 0, 0, GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE);
	gdk_pixbuf_save(pixbuf,filename,"png", &error,NULL);
}
#endif

void population_initialization() {
	/********************************************/
	/* Pointers to the agents are placed in the */
	/* vector agent[]. Each cell of the grid    */
	/* (GRID_SIZE x GRID_SIZE) is represented   */
	/* by an element of the array cell[][].     */
	/* cell[i][j] is the cell of the ith row    */
	/* and jth column. It contains a pointer to */
	/* the agent placed on it (if any).         */
	/********************************************/
	
	/* Delete each agent (if any from a         */
	/* previous iteration) to free memory       */
	for (int index=0; index<agent.size(); index++) {
		delete agent[index];
	}
	agent.clear();

	/* Create agents (one in each cell)         */
	for (int i=0; i<GRID_SIZE; i++) {
		for (int j=0; j<GRID_SIZE; j++) {
			agent.push_back(new Agent());
			cell[i][j] = agent.back();
		}
	}

	/* give agents an opinion and build the social network */
	double x_coord, y_coord;
	for (int i=0; i<GRID_SIZE; i++) {
		for (int j=0; j<GRID_SIZE; j++) {
			/* opinion */
			cell[i][j]->opinion.clear();
			cell[i][j]->last_opinion.clear();
			for(int c=0; c<OPINION_DIMENSIONS; c++) {
				cell[i][j]->opinion.push_back(gsl_rng_uniform(r));
				cell[i][j]->last_opinion.push_back(cell[i][j]->opinion.back());
			}

			/* network */
			#if (MOORE_RADIUS!=INF)
			/* Moore neighbourhood */
			cell[i][j]->relation.clear();
			for (int dx=-MOORE_RADIUS; dx<=MOORE_RADIUS; dx++) {
				for (int dy=-MOORE_RADIUS; dy<=MOORE_RADIUS; dy++) {
					if(cell[periodicBC(i+dx)][periodicBC(j+dy)]!=0) {
						cell[i][j]->relation.push_back(cell[periodicBC(i+dx)][periodicBC(j+dy)]);
					}
				}
			}
			#else
			/* infinite neighbourhood */
			cell[i][j]->relation.clear();
			for (int index=0; index<agent.size(); index++) {
				cell[i][j]->relation.push_back(agent[index]);
			}
			#endif
		}
	}
	#if (ENABLE_SMALLWORLD==1)
	/* Small-World Network: rewire Moore neighbourhood */
	int shift;
	Agent* chosen_neighbour;
	Agent* current_neighbour;
	for (int i=0; i<GRID_SIZE; i++) { /* loop on the grid cells */
		for (int j=0; j<GRID_SIZE; j++) {
			for (int dx=-MOORE_RADIUS; dx<=MOORE_RADIUS; dx++) {
				shift= ( dx<0 ? 0 : -1);
				/* loop on half of the neighbours (so that each link is tested for rewiring only once): */
				for (int dy=-MOORE_RADIUS; dy<=dx+shift; dy++) { 
					current_neighbour=*find(cell[i][j]->relation.begin(), cell[i][j]->relation.end(), cell[periodicBC(i+dx)][periodicBC(j+dy)]);
					/* test wether link has not been removed by a previous rewiring, and exclude oneself: */
					if ( current_neighbour!= *(cell[i][j]->relation.end()) && current_neighbour!=cell[i][j]) {
						if (gsl_rng_uniform(r) < double(SMALLWORLD_P)) {
							/* choose the new neighbour (forbid duplicate links) */
							chosen_neighbour=agent[(int)floor(gsl_rng_uniform(r)*double(agent.size()))];
							while (find(cell[i][j]->relation.begin(), cell[i][j]->relation.end(), chosen_neighbour)!=cell[i][j]->relation.end()) {
								chosen_neighbour=agent[(int)floor(gsl_rng_uniform(r)*double(agent.size()))];
							}
							/* destroy link from current neighbour to here*/
							current_neighbour->relation.erase(find(current_neighbour->relation.begin(), current_neighbour->relation.end(), cell[i][j]));
							/* replace link from here to current neighbour by a link from here to chosen neighbour*/
							replace(cell[i][j]->relation.begin(), cell[i][j]->relation.end(), current_neighbour, chosen_neighbour);
							/* create new link from chosen neighbour to here */
							chosen_neighbour->relation.push_back(cell[i][j]);
						}
					}
				}
			}
		}
	}
	#endif
}

void iterate() {
	#if (RANDOM_SEQUENTIAL_UPDATE==1)
	/* choose an agent randomly: */
	int i=(int)floor(gsl_rng_uniform(r)*double(agent.size()));
	gsl_matrix * neighbour_opinion = gsl_matrix_alloc(agent[i]->relation.size(),OPINION_DIMENSIONS);
	/* loop on the neighbourhood to collect neighbours' opinions */
	for (int k=0; k<agent[i]->relation.size(); k++) {
		if ((agent[i]->relation)[k]!=0) {
			for (int c=0; c<OPINION_DIMENSIONS; c++) {
				gsl_matrix_set(neighbour_opinion,k,c,(agent[i]->relation).at(k)->opinion.at(c));
			}
		}
	}
	agent[i]->self_categorize(neighbour_opinion);
	agent[i]->actualize();
	gsl_matrix_free(neighbour_opinion);
	#else
	/* loop on the agents */
	for (vector<Agent*>::iterator current_agent=agent.begin(); current_agent<agent.end(); current_agent++) {
		/* matrix of neighbours' opinions */
		gsl_matrix * neighbour_opinion = gsl_matrix_alloc((*current_agent)->relation.size(),OPINION_DIMENSIONS); 
		/* loop on the neighbourhood to collect neighbours' opinions */
		for (int k=0; k<(*current_agent)->relation.size(); k++) {
			if (((*current_agent)->relation)[k]!=0) {
				for (int c=0; c<OPINION_DIMENSIONS; c++) {
					gsl_matrix_set(neighbour_opinion,k,c,((*current_agent)->relation).at(k)->opinion.at(c));
				}
			}
		}
		(*current_agent)->self_categorize(neighbour_opinion);
		gsl_matrix_free(neighbour_opinion);
	}
	for (vector<Agent*>::iterator current_agent=agent.begin(); current_agent<agent.end(); current_agent++) {
		(*current_agent)->actualize();
	}
	#endif
}

#ifdef ENABLE_GRAPHICAL_MODE
void show_info_popup() {
	info_popup = gtk_message_dialog_new (GTK_WINDOW (NULL),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_INFO,GTK_BUTTONS_NONE,
		"Available key commands:\n\ni  -\tIterate\nr  -\tReset simulation\nn  -\tprints the Number of different opinions in population\nm  -\tprints opinion Mean\nv  -\tprints opinion Variance\no  -\tprints Opinion list\nh  -\tprints opinion Histogram\nl  -\tprints the mean Last change of opinion\ns  -\tSave a .png snapshot of the opinion window\nq  -\tQuit program\n\n---------------------------------\n\nF1 or ?  -\topen this window");
	gtk_signal_connect (GTK_OBJECT (info_popup), "key_press_event", GTK_SIGNAL_FUNC (key_event),NULL);
	gtk_widget_show(info_popup);
}

/* managing key commands in graphical mode */
void key_event (GtkWidget *widget, GdkEventKey *event, gpointer data ) {
	switch (event->keyval) {
		case GDK_i: iterate();iteration++; cout<<"iteration: "<<iteration<<endl; draw_all(GTK_WIDGET (drawing_area)); draw_all_grad(GTK_WIDGET (drawing_area_grad)); break;
		case GDK_r: population_initialization();iteration=0; draw_all(GTK_WIDGET (drawing_area));draw_all_grad(GTK_WIDGET (drawing_area_grad)); cout<<"---\npopulation reinitialized\n---"<<endl; break;
		case GDK_n: cout<<"iteration: "<<iteration<<", nb of different opinions in population: "<<number_of_different_opinions()<<endl; break;
		case GDK_m: cout<<"iteration: "<<iteration<<", opinion mean: "<<opinion_mean()<<endl; break;
		case GDK_v: cout<<"iteration: "<<iteration<<", opinion variance: "<<opinion_var()<<endl; break;
		case GDK_o: cout<<"iteration: "<<iteration<<", opinion list: "<<opinion_list()<<endl; break;
		case GDK_h: cout<<"iteration: "<<iteration<<", histogram of opinions ("<<HISTOGRAM_CLASS_NUMBER<<" opinion classes): "<<opinion_histogram(HISTOGRAM_CLASS_NUMBER)<<endl; break;
		case GDK_l: cout<<"iteration: "<<iteration<<", mean last change of opinion: "<<mean_last_change_of_opinion()<<endl; break;
		case GDK_question: show_info_popup(); break;
		case GDK_F1: show_info_popup(); break;
		case GDK_s: save_image(); cout<<"iteration: "<<iteration<<", snapshot saved"<<endl; break;
		case GDK_q: gtk_main_quit();
	}
}
#endif

int main( int argc, char *argv[] ) {
	/* initialize the GSL pseudo-random number generator */
	gsl_rng_env_setup();
	TRand=gsl_rng_mt19937;
	r=gsl_rng_alloc(TRand);
	gsl_rng_set(r,(unsigned int)time((time_t *)NULL));

	/* if option '-g' present and                        */
	/* ENABLE_GRAPHICAL_MODE==1, run the program in      */
	/* graphical mode. */
	#ifdef ENABLE_GRAPHICAL_MODE
	if (argc==1){ /* no options called */
		cout<<"Usage:\n itselfcat -g           \tfor graphical mode\n itselfcat -o output.dat\tfor iterative mode (results are saved in file output.dat)\n";
		return(0);
	}
	if (strcmp(argv[1], "-g")!=0 && strcmp(argv[1], "-o")!=0) { /* wrong options called */
		cout<<"Usage:\n itselfcat -g           \tfor graphical mode\n itselfcat -o output.dat\tfor iterative mode (results are saved in file output.dat)\n";
		return(0);
	}
	
	if (strcmp(argv[1], "-g")==0) {
		a=A;
		w=W;
		iteration=0;
		population_initialization();
		
		/* creation of the windows */
		gtk_init(&argc, &argv);
	
		/* opinion window */
		window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW (window),"Opinions");
		gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (delete_event), NULL);
		gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
		gtk_signal_connect (GTK_OBJECT (window), "key_press_event", GTK_SIGNAL_FUNC (key_event),NULL);
		gtk_container_set_border_width (GTK_CONTAINER (window), 10);
		vbox = gtk_vbox_new (FALSE, 0);
		gtk_container_add (GTK_CONTAINER (window), vbox);
		gtk_widget_show (vbox);
		drawing_area =gtk_drawing_area_new();
		gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE);
		gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
		gtk_widget_show (drawing_area);
		gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",(GtkSignalFunc) expose_event, NULL);
		gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",(GtkSignalFunc) configure_event, NULL);
	
		/* gradient window */
		window_grad = gtk_window_new (GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW (window_grad),"Gradient");
		gtk_signal_connect (GTK_OBJECT (window_grad), "delete_event", GTK_SIGNAL_FUNC (delete_event), NULL);
		gtk_signal_connect (GTK_OBJECT (window_grad), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
		gtk_signal_connect (GTK_OBJECT (window_grad), "key_press_event", GTK_SIGNAL_FUNC (key_event),NULL);
		gtk_container_set_border_width (GTK_CONTAINER (window_grad), 10);
		vbox_grad = gtk_vbox_new (FALSE, 0);
		gtk_container_add (GTK_CONTAINER (window_grad), vbox_grad);
		gtk_widget_show (vbox_grad);
		drawing_area_grad =gtk_drawing_area_new();
		gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area_grad), GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE);
		gtk_box_pack_start (GTK_BOX (vbox_grad), drawing_area_grad, TRUE, TRUE, 0);
		gtk_widget_show (drawing_area_grad);
		gtk_signal_connect (GTK_OBJECT (drawing_area_grad), "expose_event",(GtkSignalFunc) expose_event_grad, NULL);
		gtk_signal_connect (GTK_OBJECT(drawing_area_grad),"configure_event",(GtkSignalFunc) configure_event_grad, NULL);
	
	
		/* info popup */
		show_info_popup();
		gtk_widget_show(window_grad);
		gtk_widget_show(window);
	
		draw_all_grad(GTK_WIDGET (drawing_area_grad));
		draw_all(GTK_WIDGET (drawing_area));
	
		gtk_main ();
		return(1);
	}
	#endif
	
	if (argc==1){ /* no options called*/
		cout<<"Usage:\n itselfcat -o output.dat\tfor iterative mode (results are saved in file output.dat)\n";
		return(0);
	}
	if (strcmp(argv[1], "-o")!=0) { /* wrong options called */
		cout<<"Usage:\n itselfcat -o output.dat\tfor iterative mode (results are saved in file output.dat)\n";
		return(0);
	}
	else if (strcmp(argv[1], "-o")==0)
	{
		/* option '-o', run the program in iterative mode. */
		ostringstream text;
		ofstream out(argv[2]);
		time_t start, end;

		#if (HISTOGRAM_OUT==1)
		char histogram_classes[HISTOGRAM_CLASS_NUMBER*20]="";
		char buffer[20];
		sprintf(histogram_classes,"\t[%6.5f;%6.5f", 0.0, 1.0/double(HISTOGRAM_CLASS_NUMBER));
		for (int c=1; c<HISTOGRAM_CLASS_NUMBER; c++) {
			sprintf(buffer, "[\t[%6.5f;%6.5f", double(c)/double(HISTOGRAM_CLASS_NUMBER), double(c+1.0)/double(HISTOGRAM_CLASS_NUMBER));
			strcat(histogram_classes, buffer);
		}
		strcat(histogram_classes, "]");
		#endif
		#if (SNAPSHOT_OUT==1)
		gtk_init(&argc, &argv);
		window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW (window),"Opinions");
		gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (delete_event), NULL);
		gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
		gtk_container_set_border_width (GTK_CONTAINER (window), 10);
		vbox = gtk_vbox_new (FALSE, 0);
		gtk_container_add (GTK_CONTAINER (window), vbox);
		gtk_widget_show (vbox);
		drawing_area =gtk_drawing_area_new();
		gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), GRID_SIZE*CELLSIZE, GRID_SIZE*CELLSIZE);
		gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
		gtk_widget_show (drawing_area);
		gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",(GtkSignalFunc) expose_event, NULL);
		gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",(GtkSignalFunc) configure_event, NULL);
		gtk_widget_show(window);
		#endif
		
		/* print info about what is going on to the standard output */
		printf("\nParameter 'a' ranges from %f to %f, with step %f\n", FIRST_A, LAST_A, STEP_A);
		printf("Parameter 'w' ranges from %f to %f, with step %f\n", FIRST_W, LAST_W, STEP_W);
		printf("Running %d experiments (of %d iterations) for each combination of the parameters\n", NUMBER_OF_EXPERIMENTS, LAST_ITERATION);
		total_experiments_number=0;
		/* compute total number of experiments (explicitely to avoid some strange things that can happen when comparing two 'double')*/
		for (a=FIRST_A; a<=LAST_A+1e-9; a+=STEP_A) {
			for (w=FIRST_W; w<=LAST_W+1e-9; w+=STEP_W) {
				experiment=0;
				while (experiment < NUMBER_OF_EXPERIMENTS) {
					experiment++;
					total_experiments_number++;
				}
			}
		}
		printf("\nTotal: %d experiments to be computed.\n\n",total_experiments_number);
		cout<<"----------\nOutput in file "<<argv[2]<<" :\n\n"
		#if (MEAN_LAST_OPINION_CHANGE_OUT==1)
		<<"mean_last_change_of_opinion\n"
		#endif
		#if (NB_OF_DIFFERENT_OPINIONS_OUT==1)
		<<"number_of_different_opinions\n"
		#endif
		#if (MEAN_OUT==1)
		<<"opinion_mean\n"
		#endif
		#if (VARIANCE_OUT==1)
		<<"opinion_var\n"
		#endif
		#if (HISTOGRAM_OUT==1)
		<<"opinion_histogram("<<HISTOGRAM_CLASS_NUMBER<<")\n"
		#endif
		#if (OPINION_LIST_OUT==1)
		<<"opinion_list("<<GRID_SIZE*GRID_SIZE<<")\n"
		#endif
		;
		time(&start);
		printf("----------\n\nStarting %s", ctime(&start));
		cout<<"Done   0% [                              ]";
		cout.flush();
		
		/* print headings in output file*/
		/* first line : */
		text<<"Iterated self-categorization, "<<GRID_SIZE*GRID_SIZE<<" ("<<GRID_SIZE<<"*"<<GRID_SIZE<<") agents, [0,1]-uniform initial opinions distribution, "<<
		#if (MOORE_RADIUS==INF)
		"inf"
		#else
		MOORE_RADIUS
		#endif
		<<"-radius Moore neighbourhood, periodic boundary conditions";
		#if (ENABLE_SMALLWORLD==1)
		text<<", small-world p="<<SMALLWORLD_P;
		#endif
		#if (RANDOM_SEQUENTIAL_UPDATE==1)
		text<<", random sequential update";
		#else
		text<<", parallel update";
		#endif
		text<<endl;
		out<<text.str();
		text.str("");

		/* second line : */
		text<<"a\tw\titeration"
		#if (MEAN_LAST_OPINION_CHANGE_OUT==1)
		<<"\tmean_last_change_of_opinion"
		#endif
		#if (NB_OF_DIFFERENT_OPINIONS_OUT==1)
		<<"\tnumber_of_different_opinions"
		#endif
		#if (MEAN_OUT==1)
		<<"\topinion_mean"
		#endif
		#if (VARIANCE_OUT==1)
		<<"\topinion_var"
		#endif
		#if (HISTOGRAM_OUT==1)
		<<histogram_classes
		#endif
		#if (OPINION_LIST_OUT==1)
		<<"\topinion_list("<<GRID_SIZE*GRID_SIZE<<")"
		#endif
		<<"\n";
		out<<text.str();
		out.flush(); /*physically writes the line in the file*/
		text.str("");
		computed_experiments=0;
		percent_computed_experiments=0;

		/* main iteration loop */
		for (a=FIRST_A; a<=LAST_A+1e-9; a+=STEP_A) {
			for (w=FIRST_W; w<=LAST_W+1e-9; w+=STEP_W) {
				experiment=0;
				while (experiment < NUMBER_OF_EXPERIMENTS) {
					population_initialization();
					nothing_happened=false;
					iteration=0;
					while (iteration <= LAST_ITERATION) {
						if (iteration%STEP_OUTPUT == 0 && iteration >= FIRST_OUTPUT) { /*output of the results*/
							text.str("");
							text<<a<<"\t"<<w<<"\t"<<iteration
							#if (MEAN_LAST_OPINION_CHANGE_OUT==1)
							<<"\t"<<mean_last_change_of_opinion()
							#endif
							#if (NB_OF_DIFFERENT_OPINIONS_OUT==1)
							<<"\t"<<number_of_different_opinions()
							#endif
							#if (MEAN_OUT==1)
							<<"\t"<<opinion_mean()
							#endif
							#if (VARIANCE_OUT==1)
							<<"\t"<<opinion_var()
							#endif
							#if (HISTOGRAM_OUT==1)
							<<"\t"<<opinion_histogram(HISTOGRAM_CLASS_NUMBER)
							#endif
							#if (OPINION_LIST_OUT==1)
							<<"\t"<<opinion_list()
							#endif
							<<"\n";
							out<<text.str();
							out.flush();
							#if (SNAPSHOT_OUT==1)
							draw_all(GTK_WIDGET (drawing_area));
							save_image();
							#endif
						}
						#if (RANDOM_SEQUENTIAL_UPDATE==0)
						/* iterate only if a stationary state was not reached to spare computing time */
						if(!nothing_happened) {
							nothing_happened=true;
							iterate();
						}
						#else
						iterate();
						#endif
						iteration++;
					}
					experiment++;
					computed_experiments++;
					percent_computed_experiments=100*(double)computed_experiments/(double)total_experiments_number;
					cout<<"\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
					printf("%3d%% ", (int)percent_computed_experiments);
					int bar_length=(int)(percent_computed_experiments*30/100);
					if (bar_length==0) {bar_length=1;}
					string progress_bar_done (bar_length-1,'=');
					string progress_bar_undone (30-bar_length,' ');
					cout<<"["<<progress_bar_done<<">"<<progress_bar_undone<<"]";
					cout.flush();
				}
			}
		}
		out.close();
		time(&end);
		printf("\nEnded %s", ctime(&end));
		int dif, hours, minutes, seconds;
		dif=(int)difftime(end,start);
		hours=(int)dif/3600;
		minutes=(int)(dif/60)-hours*60;
		seconds=dif-60*minutes-3600*hours;
		printf("Total calculation time: %3d hours, %2d minutes, %2d seconds.\n\n", hours, minutes, seconds);
	}
	gsl_rng_free(r);
	return(0);
}

